@@ -2,10 +2,17 @@
 """
 Decipher Beacon 2.0 API access library
 """
+from __future__ import print_function
+
+import hashlib
+
+import six
+
+try:
+    from ConfigParser import RawConfigParser, NoSectionError, DuplicateSectionError, NoOptionError
+except ImportError:
+    from configparser import RawConfigParser, NoSectionError, DuplicateSectionError, NoOptionError
 
-# The API version we'll be accessing
-from ConfigParser import RawConfigParser, NoSectionError, DuplicateSectionError, NoOptionError
-from functools import partial
 import hmac
 import json
 import os, requests
@@ -40,14 +47,27 @@ class BeaconAPI(object):
     section = 'main'    # name of section in INI file we are using
     keySource = None
     timeout = None
+    verifySSL = True
+    impersonate = None
 
-    def __init__(self, host=None, key=None):
+    def __init__(self, host=None, key=None, retry=None):
         self.host, self.key = host, key
         self.session = requests.Session()
+        self.session.verify = self.verifySSL
+        if retry is not None:
+            from requests.packages.urllib3.util.retry import Retry
+            from requests.adapters import HTTPAdapter
+            retry = Retry(total=retry, read=retry, connect=retry,
+                          backoff_factor=1.0,
+                          status_forcelist=(500, 502, 503, 504),
+                          method_whitelist=('POST',))
+            adapter = HTTPAdapter(max_retries=retry)
+            self.session.mount('https://', adapter)
+            self.session.mount('http://', adapter)
 
     def _debug(self, what):
         if self.verbose:
-            print >> sys.stderr, what
+            print(what, file=sys.stderr)
 
 
     def login(self, key, host=DEFAULT_HOST):
@@ -104,7 +124,7 @@ class BeaconAPI(object):
         if username:
             parser.set(section, "username", username)
         parser.write(open(self.inifile, "w"))
-        os.chmod(self.inifile, 0600)
+        os.chmod(self.inifile, 0o600)
 
 
     def _ensureKey(self):
@@ -132,7 +152,8 @@ class BeaconAPI(object):
 
                 # convert key to API key
                 username = pwd.getpwuid(os.getuid()).pw_name
-                self.key = 'local %s %s' % (username, hmac.new(ekey, username).hexdigest())
+                self.key = 'local %s %s' % (username, six.ensure_str(
+                    hmac.new(six.ensure_binary(ekey), six.ensure_binary(username), digestmod=hashlib.md5).hexdigest()))
                 self.keySource = 'local'
             else:
                 self._debug("+ Environment unset, using INI file section %s" % self.section)
@@ -158,13 +179,16 @@ class BeaconAPI(object):
     def _requestAuthHeaders(self):
         if self.key.startswith('session '):
             _, formkey, rest = self.key.split()
-            return {"x-apikey" : 'session %s' % formkey, "Cookie" : 'BEACON_LOGIN="%s"' % rest}
+            d = {"x-apikey" : 'session %s' % formkey, "Cookie" : 'BEACON_LOGIN="%s"' % rest}
         else:
-            return {"x-apikey" : self.key}
+            d = {"x-apikey" : self.key}
+        if self.impersonate:
+            d['x-impersonate'] = self.impersonate
+        return d
 
-    def do(self, action, name, args, async=False):
+    def do(self, action, name, args):
         "Perform action"
-        if args.pop('meta', None):
+        if args.pop('__meta', None):
             return (dict(
                 api = '/api/%s/%s' % (self.version, name),
                 method = action.upper(),
@@ -179,7 +203,7 @@ class BeaconAPI(object):
         if action == 'get':
             body = kw['params'] = args
         else:
-            body = kw['data'] = json.dumps(args, indent=1)
+            body = kw['data'] = json.dumps(args, indent=1,cls=JSONEncoder)
         if self.timeout: kw['timeout'] = self.timeout
 
 
@@ -192,15 +216,12 @@ class BeaconAPI(object):
 
         if self.xml:
             headers['accept'] = 'application/xml'
+        kw['verify'] = self.verifySSL
 
         headers.update(self.headers)
-        if async:
-            import grequests
-            return grequests.AsyncRequest(action, url, headers=headers, **kw)
-
         try:
-            r = self.session.request(action, url, verify=True, headers=headers, **kw)
-        except requests.ConnectionError, e:
+            r = self.session.request(action, url, headers=headers, **kw)
+        except requests.ConnectionError as e:
             raise BeaconAPIException(code=500, message="Could not connect to server (%s): %s" % (url, e))
         self._debug('<< %s %s' % (r.status_code, r.reason))
         if 'x-typehint' in r.headers:
